;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xad5e63f332817e35c8c271422ad949a5 . "./code.cg2js/cg-status-bar.fasl")
(#xaa9a7cfb8f9c9df95cc82228c6dae03d . "./code.cg2js/cg-hotspot.fasl")
(#xcf5b42f2164021140bdf2e702c485833 . "./code.cg2js/cg-toolbar.fasl")
(#xae4216ab67122e9e552744d4500f4180 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x7f8925626481d0118701223d3e72501b . "./code.cg2js/cg-tooltip.fasl")
(#x3520165639dba5432818eb0df1c3d312 . "./code.cg2js/cg-calendar.fasl")
(#x8b7ac901915e50ecf310265e45c22777 . "./code.cg2js/cg-item-list.fasl")
(#x38b97fc580e3a1e8ce566973ff6559c5 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xffc22243cf8c2f95cea15eb83fab4b8a . "./code.cg2js/cg-trackbar.fasl")
(#xcb5841fa13284ed80d52c9ac5d547947 . "./code.cg2js/cg-class-grid.fasl")
(#xa43526a08a0d60f3ed792a3b338e8ed2 . "./code.cg2js/cg-dialog-item.fasl")
(#xb31adf907637bd0d5a3e4f90bc3f6a4d . "./code.cg2js/cg-comtab.fasl")
(#x67c00a279473d2da0636113df401554d . "./code.cg2js/cg-lisp-text.fasl")
(#x76029b4030f3aa81ba87cb05e10e0fd4 . "./code.cg2js/cg-button.fasl")
(#xa5955c12a4a50c274a941412fad54b87 . "./code.cg2js/cg-printing.fasl")
(#x256e4f2b56974dddb85121afb67b2071 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xc1c56922e9508e3eb6cb3b22b12d39a8 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xfbb419aefa67ad18fb3a7d69bf17d4a6 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x504bb987dbccb5857eb11d6a43992ade . "./code.cg2js/cg-selected-object.fasl")
(#xc89b336ba381f61e6210e6ced1bfb695 . "./code.cg2js/cg-text-or-combo.fasl")
(#x8e51c1be953b0671cb78ab03413eb898 . "./code.cg2js/cg-graphics-context.fasl")
(#xcf49597bce58ad1ebc67e14d7006c222 . "./code.cg2js/cg-edit-in-place.fasl")
(#x6cc7bdce47a5ca252caa2d9eccc8a8ce . "./code.cg2js/cg-string-dialog.fasl")
(#xc7791f403ac781f950ab77613e4795d0 . "./code.cg2js/cg-list-view.fasl")
(#x91e7e52680c25f35d83575978b0a6e11 . "./code.cg2js/cg-intersect.fasl")
(#x1b82330fa4536c3abbf1404dfc402c30 . "./code.cg2js/cg-text-widget.fasl")
(#x4475a226daec64506cadc850e891de82 . "./code.cg2js/cg-directory-dialog.fasl")
(#xad9ccbc476b0692656b9e8cf79e3fd60 . "./code.cg2js/cg-file-dialog.fasl")
(#x2df058699dd028b6b57dd92e1d58a1a5 . "./code.cg2js/cg-dde-utils.fasl")
(#x53b13a7c9cc751e836cf885b412ca7a8 . "./code.cg2js/cg-nodes.fasl")
(#x78362eaea1dd23086e621e5524561fe8 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xceb51d7d604576d054d38b97200ab1e3 . "./code.cg2js/cg-up-down-control.fasl")
(#xe03620347684cbe1037d7f7ff6452900 . "./code.cg2js/cg-get-position.fasl")
(#xa0403aa83848637a5132598844038fd5 . "./code.cg2js/cg-toggling-widget.fasl")
(#x82b5cbda87f4be62fa8fdfd17e68c81c . "./code.cg2js/cg-acache.fasl")
(#xb716d3f97a1f88293b62a2aafc50b2a7 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xeb1bf35492060553c0e552ab27bc6393 . "./code.cg2js/cg-base.fasl")
(#x37dea74d496d79b103541b6b47e803aa . "./code.cg2js/cg-drag-and-drop.fasl")
(#x3c03028a8d57db4f4a0f207785770fde . "./code.cg2js/cg-find-string-dialog.fasl")
(#xcc0b50ef4b601047e95e28b56222f9f6 . "./code.cg2js/cg-property.fasl")
(#x0e6b3972d98669a37ba7e3c911cad584 . "./code.cg2js/cg-editable-text.fasl")
(#x7297d5451efce73ff7901bbb47461b8f . "./code.cg2js/cg-lamp.fasl")
(#x194bbc047fedfd660d97967aff613b74 . "./code.cg2js/cg-progress-indicator.fasl")
(#x78efcbdb0fa510870e3a5bd0848b49f2 . "./code.cg2js/cg-chart-widget.fasl")
(#xfca0fe90e200959c2da47e04cd0acfcf . "./code.cg2js/cg-common-control.fasl")
(#xd50794b9aa9b03b08ac4023cd2bb0d1e . "./code.cg2js/cg-font-dialog.fasl")
(#x51ea5138cac8edae510dbdd1257dbd46 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x9307429be00175ecfb676e0a153a8b74 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x38c6496cb62774a47d5831641b0a2b9d . "./code.cg2js/cg-os-window.fasl")
(#xec494cf59db6365ef9112f76eb06f437 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x65ae6a28e965178f9b67318b502a6aa5 . "./code.cg2js/cg-pixmap.fasl")
(#x367faa64074e6d4f88bafa7cb4b6d1d0 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x0c08479d4d9d32596bc2e12f7bd1c4cb . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x579c44a6396dc47d4e833dd3e33e1fc7 . "./code.cg2js/cg-scaling-stream.fasl")
(#xd5d64035f6cf8c87c7ffe80c0e521f20 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xc7c559c5a64bd2bc5fd9eaaeb690136c . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x3abc82b8ee3fd234fed7233be649264e . "./code.cg2js/cg-timer.fasl")
(#xd4036615ff5c5526224a8c1c55365103 . "./code.cg2js/cg-scroll-bar.fasl")
(#x2d29cef2a6662df8e21c31139252fa6a . "./code.cg2js/cg-caret.fasl")
(#xdd54be13d578a1cf57166cb0ddc3c090 . "./code.cg2js/cg-template-string.fasl")
(#x478f65a723defe768dccacea29d0f1a8 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xb44d72846130a09c86bce79ebb19193f . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xd0a6be2896817462d88fc6690ad06c8f . "./code.cg2js/cg-combo-box.fasl")
(#x3d5c244dd9ecf49ebd2e4180f67a0f6c . "./code.cg2js/cg-text-edit-pane.fasl")
(#x2eabc56c015baf709029fe166ec318dd . "./code.cg2js/cg-multi-picture-button.fasl")
(#x8504b5ce34858dd61f830e9e66090a2d . "./code.cg2js/cg-menu-tooltip.fasl")
(#x25ed610fb65f465d917983e8e3a9dad1 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xfe138ab557aa3adc2b26c819d6ab3785 . "./code.cg2js/cg-player.fasl")
(#xdaa59af3ec44a4699d51dcca26e20c7f . "./code.cg2js/cg-outline.fasl")
(#xdcbf2b4562553ad1d165087d8e131477 . "./code.cg2js/cg-class-support.fasl")
(#x794c0d6b71bae913b8eaf05349763f54 . "./code.cg2js/cg-lettered-menu.fasl")
(#x7807cc1016101d44c7d27f47c78888aa . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x297dfa6456356862c5bc66814fa365c4 . "./code.cg2js/cg-fill-texture.fasl")
(#xf347407bb2c8577b904384369069de19 . "./code.cg2js/cg-lisp-widget.fasl")
(#xc4acb75425ddf5c5db2c62119b0a71f6 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x46c946ae35b74312a827e57297d6dc05 . "./code.cg2js/cg-clipboard.fasl")
(#x1ae3904e23dae91d7af1231378d8cce8 . "./code.cg2js/cg-wrap-string.fasl")
(#x0f0a60c29a84e2e5a640fdda324a9d47 . "./code.cg2js/cg-os-widget.fasl")
(#xef850dfa34f0623bc16f63bf608d038f . "./code.cg2js/cg-check-box.fasl")
(#x9e5ac1c1a74cbad69db15adc2c5104c6 . "./code.cg2js/cg-choice-list.fasl")
(#x9e83357106f9fc7fdd19e2ce99898a49 . "./code.cg2js/cg-layout.fasl")
(#x3e06c38c5fe97c9e02e4bf7f81826c68 . "./code.cg2js/cg-group-box.fasl")
(#x329e7b872a635549fc3af0664cc037f9 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xda31d85182d01947275d3906bda43343 . "./code.cg2js/cg-radio-button.fasl")
(#xb254917c5b2b148ee99736cfb521ba2b . "./code.cg2js/cg-tab-control.fasl")
(#xefdc250da6391a33e38c9feb7a546997 . "./code.cg2js/cg.fasl")
(#xd1beb01e99c2dbead2a67c40c5518e64 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xf4b9b4250c9be4a4c5b4b7afabc30683 . "./code.cg2js/cg-dropping-outline.fasl")
(#x68a4a30f61d851897a13fe050c2443a4 . "./code.cg2js/cg-get-pixmap.fasl")
(#x562da8277b17b6c5594091e91fc0261c . "./code.cg2js/cg-grid-widget.fasl")
(#xb51514f4bee2795f780ec88ff5d6cbf8 . "./code.cg2js/cg-utility-dialog.fasl")
(#xbff1e27581752358a8cf3d0e1e2b3edf . "./code.cg2js/cg-static-text.fasl")
(#x366f0a54b37095a6096316a9ef6211a7 . "./code.cg2js/cg-class-slot-grid.fasl")
(#xba0289d8b18dbd15f784d535f4af8b66 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x95ee4a22c253019061851808e83c2051 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xfb479fbfa04c93de8c13617d8d3cc52b . "./code.cg2js/cg-bitmap-pane.fasl")
(#xa7de89c36c226c367f624eec63b33fb0 . "./code.cg2js/cg-plot-widget.fasl")
(#x428eadee5f378a6267887259f5a8def0 . "./code.cg2js/cg-color-dialog.fasl")
(#x8a68f7a67b2a5c8afb6c57ab954613a0 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x2917bc8d0b9d688af1ee0d88ea699b02 . "./code.cg2js/cg-icon.fasl")
(#xee973638a6d006435982f720023ca977 . "./code.cg2js/cg-dev.fasl")
(#x6fb1adf99b19bc5f1de9ea4a4b0c7f3e . "./code.cg2js/cg-choose-printer.fasl")
(#xe0510c6caab69f744db51b3f16ee8757 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x4761f86e4cf61bd1b9dd843be18735ce . "./code.cg2js/cg-menu.fasl")
(#x92e0ad0d76cfceab4435e6f72c155787 . "./code.cg2js/cg-yes-no-list.fasl")
(#x53cc030d142e0d90c302f8d0d42f584b . "./code.cg2js/cg-palette.fasl")
(#xf2997c855b7150d60437d9466201a957 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x5d1bfcb7c83fb76dd3afacea5d2824a9 . "./code.cg2js/cg-message-dialog.fasl")
(#x434d4a462c829f88a569c87daa3c5a0d . "./code.cg2js/cg-shortcut-menu.fasl")
(#xcb0f6252835d53174936d725ef09abd0 . "./code.cg2js/cg-split-bar.fasl")
(#xac700e9556913459d73367b6833bb2c5 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x2ade8bff854283cd456a3f735644ee0c . "./code.cg2js/cg-web-browser.fasl")
(#x6d1e9636092f8e290bc05f19d0870013 . "./code.cg2js/cg-project-window.fasl")
(#x45fd2ec1401eb0f03ab7d41ef2cf82ca . "./code.cg2js/cg-picture-widget.fasl")
(#xc30705b0ba0738c538b3c814a6a3ad50 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x43effff666489ae58ef69bbd19af9b0b . "./code.cg2js/cg-object-editor.fasl")
(#x295c82282abba43c21853e3c9a397db5 . "./code.cg2js/cg-paren-matching.fasl")
(#xf27dd90d78e1765d5282ef9481c3b229 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x027316af4c0f121b0c24cdcf9345143c . "./code.cg2js/cg-drawable.fasl")
(#x2c14c4fe591edce182a531c31454ac12 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x94a918617ed190344f6ceb3e31cbad05 . "./code.cg2js/cg-html-widget.fasl")
(#xc0f3b7147c99f51895758a8606299a2b . "./code.cg2js/cg-curve.fasl")
(#x1cc12b1229b1fc94d9f2af01d0cf2c11 . "./code.cg2js/cg-yes-no-string.fasl")
(#xb111b39e775c923a79e6b1d13ec421b8 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x6afd9b4c25116c79cc64bae68c048c7e . "./code.cg2js/cg-html-dialog.fasl")
(#x7943670bb51eb4dcd06578280373d111 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xa4a6059f19d6c0aecc5bf07aaaba39ef . "./code.cg2js/cg-tray.fasl")
(#xf6db90a66f97974231e34e1944036619 . "./code.idejs/ide-apropos.fasl")
(#x11c0a3afc56a42454d025b227f3c045a . "./code.idejs/ide-window-list.fasl")
(#x0e96d154d3417c443164aecfe280e22b . "./code.idejs/ide-class-browser.fasl")
(#x0d806bd895f9c29610eeb2340987d9a2 . "./code.idejs/ide-base.fasl")
(#x8662d7e2ca5d8344eafb4bba7d9b835f . "./code.idejs/ide-process-browser.fasl")
(#xe521ff6d37f5fb15692eb2f93c2f4d22 . "./code.idejs/ide-acache.fasl")
(#xed83f5660bb153f9cd49596f52fb618a . "./code.idejs/ide-navigator.fasl")
(#x856945198e38d9338b04851b9fadd6cf . "./code.idejs/ide-trace.fasl")
(#x9396f1defa90c6576ba159ac1945c3c1 . "./code.idejs/ide-project.fasl")
(#x0beda389bb77367f5a1c128f59b4c13f . "./code.idejs/ide-patch-dialog.fasl")
(#x76cbb14c0506f0159f0fcb50871d8866 . "./code.idejs/ide-grapher.fasl")
(#x282d119ef00804f3293c84f16e43cd5b . "./code.idejs/ide-dev.fasl")
(#x5177a78c7b84cd409b6414e32dd85e79 . "./code.idejs/ide-keyshort.fasl")
(#x151283ac170374c39ca66dc36960d5f0 . "./code.idejs/ide-editor.fasl")
(#x24679022c6328dea0f9111e4b00f467f . "./code.idejs/ide-inspector.fasl")
(#x4c19eca63a257d1c08b2bfce5d771b58 . "./code.idejs/ide.fasl")
(#x60e1373cd4bbd73fddd868b8cd63e6f0 . "./code.idejs/ide-clipboard.fasl")
(#xc180f0023bd763869ead6d999bf89100 . "./code.idejs/ide-find-definitions.fasl")
(#x06a57036e6c712663ca6a9e3610df9bb . "./code.idejs/ide-form.fasl")
(#x7655269e2530d5521c406d969e7b17d7 . "./code.idejs/ide-intro.fasl")
(#xc487adc16329ce750c39537799235d09 . "./code.idejs/ide-find-in-files.fasl")
(#x52f38451bf7dd4d15f1de7eff017e650 . "./code.idejs/ide-package-list.fasl")
(#x497b739bbd662107f48e4ce637d3c04b . "./code.idejs/ide-class-grapher.fasl")
(#x9a8f50ffdba20a10156ef75920aa924b . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (format t "Incorrect checksum on file: ~a" (cdr checksum))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
